import java.util.List;
import java.util.Vector;

/**
 * Stores an adjacency matrix for a Graph data structure. 
 * @author patmore
 *
 * @param <T> - Data type of the graph's nodes, a generic type
 */
public class AdjacencyMatrix <T> {
	private int size; //adjacency matrices are always square
	//Vector of Vectors, internal vectors are one matrix row
	private Vector<Vector<Integer>> matrix;
	List<T> nodeNames;
	
	/**
	 * Make a new AdjacencyMatrix
	 * @param size - List, storing the nodes of the graph. This can be any object 
	 * implementing the List interface
	 */
	public AdjacencyMatrix(List<T> theNodes){
		size = theNodes.size();
		nodeNames = theNodes;
		matrix = new Vector<Vector<Integer>>(size);
		for(int i = 0; i < size; i++){
			Vector<Integer> aRow = new Vector<Integer>(size);
			for(int j = 0; j< size; j++){
				aRow.add(0);
			}
			matrix.add(aRow);
		}
	}
	/**
	 * sets an element in the matrix. numerical addressing
	 * @param x - int, x coordinate (column), count from zero
	 * @param y - int, y coordinate (row), count from zero
	 * @param data - int, value to set matrix cell to
	 */
	private void setElement(int x, int y, int data){
		Vector<Integer> row = matrix.get(y);
		row.set(x, data);
	}
	/**
	 * gets an element using numerical addressing. same scheme as setElement
	 * @param x - int
	 * @param y - int
	 * @return - int, the data that is in the cell
	 */
	private int getElement(int x, int y){
		Vector<Integer> row = matrix.get(y);
		return row.get(x);
	}
	/**
	 * set an element of the matrix based on which nodes it relates
	 * @param x - column label 
	 * @param y - row label
	 * @param data - value to store in matrix (int)
	 */
	public void setElement(T x, T y, int data){
		int y_index = nodeNames.indexOf(y); //node names match the row/col labels
		int x_index = nodeNames.indexOf(x);
		setElement(x_index, y_index, data);
	}
	/**
	 * get an element of the matrix based on which nodes it relates
	 * @param x - column label
	 * @param y - row label
	 * @return - int, data in the relevant cell
	 */
	public int getElement(T x, T y){
		int y_index = nodeNames.indexOf(y); //node names match the row/col labels
		int x_index = nodeNames.indexOf(x);
		return getElement(x_index, y_index);
	}
	/**
	 * get a whole row of the matrix
	 * @param y - row label to get
	 * @return - Vector<Integer>, the row in question
	 */
	public Vector<Integer> getRow(T y){
		int y_index = nodeNames.indexOf(y);
		Vector<Integer> row = matrix.get(y_index);
		return row;
	}
	/**
	 * return a string representation of the adjacency matrix for printing. I've 
	 * done some formatting to (try to) make it print reasonably
	 */
	public String toString(){

		//String out = "    "; //nodeNames.toString();
		String out = String.format("%1$15s", "");
		for(int i = 0; i < nodeNames.size(); i++){
			out += nodeNames.get(i)+"  ";
			//out += String.format("%1$15s", nodeNames.get(i));
		}
		for(int i = 0; i < matrix.size(); i++){
			//out += "\n"+nodeNames.get(i)+": "+matrix.get(i).toString();
			String matString = matrix.get(i).toString();
			out += String.format("\n %1$15s  "+matString, nodeNames.get(i));
		}
		return out;
	}
}
